#!/usr/bin/env python
import os.path
import utils, re
import html_checker
clean_suffix = 'clean'

clean_file_re = re.compile(r'clean')
def clean_and_chunk(filename, chunk=True):
    m = clean_file_re.search(filename)
    if m:
        return [filename]
    else:
        lines_out = clean_up_file(filename)
        if chunk:
            chunks_out = get_big_chunks(lines_out)
            (directory, fn) = os.path.split(filename)
            (base,ext) = os.path.splitext(fn)
            if not base:
                base = 'txt'
            # return output_chunks(os.path.join(directory,base), chunks_out)
            return output_chunks(directory, base, chunks_out)
        else:
            return [output_lines(filename, lines_out)]
    


def clean_up_file (filename):
    fh = open(filename,'r')
    lines_out = html_checker.clean_up_lines(fh)
    fh.close()
    return lines_out


def output_lines (filename, lines, infix=clean_suffix):
    (directory, fn) = os.path.split(filename)
    (base, ext) = os.path.splitext(fn)
    outfile = os.path.join(directory, "%s_%s%s" % (base,infix,ext))
    o_fh = open(outfile, 'w')
    for line in lines:
        if line: # Check for empty string => skip a line
            print >> o_fh, line
        else:
            print >> o_fh
    o_fh.close()
    return outfile

def output_chunks (chunk_dir, base, chunks):
    """
    C{chunk_dir} is a dir specific to source file
    in which chunks from this file are to be stored.
    """
    ctr = 0
    # base = '%s_%s' % (base,clean_suffix)
    # print chunk_dir
    # utils.make_dir_if_necessary(chunk_dir)
    chunk_list = [ ]
    for chunk in chunks:
        chunk_list.extend(chunk)
    chunk_list = [c.strip() for c in chunk_list]
    chunk_list = [c for c in chunk_list if c]
    full_path = os.path.join(chunk_dir,'%s%stxt' % (base,os.path.extsep))
    
    return [output_lines(full_path, chunk_list)]

def old_output_chunks (chunk_dir, chunks):
    """
    C{chunk_dir} is a dir specific to source file
    in which chunks from this file are to be stored.
    """
    ctr = 0
    chunk_dir = '%s_%s' % (chunk_dir,clean_suffix)
    # print chunk_dir
    # utils.make_dir_if_necessary(chunk_dir)
    file_list = [ ]
    for chunk in chunks:
        ctr += 1
        full_path = os.path.join(chunk_dir,'chunk_%d%stxt' % (ctr,os.path.extsep))
        output_lines(full_path, chunk)
        file_list.append(full_path)
    return file_list

# max # linmes in a chunk
chunk_threshhold = 25
# line length at which to be suspicious that this is not text.
short_line_len = 5
# Max num lines for chunk
chunk_max = 20

def break_off_chunk (line, current_chunk, result):
    # print 'b_o_c: %s %s' % (line, current_chunk)
    if line:
        current_chunk.append(line+'\n')
    result.append(current_chunk)
    return (0,0,[])


def get_big_chunks (lines):
    """
    The heart of this module.
    Chunk a file into pieces each
    of which is a reliable text segment.
    """
    result = []
    (ctr, len_tot,current_chunk) = (0, 0, [])
    for line in lines:
        ctr += 1
        line = line.strip()
        words = line.split()
        line_len = len(words)
        len_tot += line_len
        if filter_strange(line):
            continue
        if line_len > chunk_threshhold:
            # print 'chunk_threshhold exceeded: %d' % line_len
            # print line
            ## multiple lines w/o line break
            (ctr, len_tot,current_chunk) = \
                  break_off_chunk(line, current_chunk, result)
            continue
        len_avg = float(len_tot)/ctr
        if len_avg > short_line_len and line_len and \
           len_avg/line_len > 2:
            # print 'short line after sequence of long ones. scoop.'
            # print line
            # short line after sequence of long ones. scoop.
            (ctr, len_tot, current_chunk) = \
                  break_off_chunk(line, current_chunk, result)
            continue
        #if len_avg <= short_line_len and ctr > chunk_max:
        #    # reset
        #    (ctr, len_tot,current_chunk) = (0, 0, [])
        if len_avg > short_line_len and ctr > chunk_max:
            # print 'chunk max exceeded'
            # print line
            (ctr, len_tot, current_chunk) = \
                  break_off_chunk(line, current_chunk, result)
            continue
        if line_len == 0:
            if len(current_chunk) > 4 and len_avg > short_line_len:
                (ctr, len_tot,current_chunk) = \
                      break_off_chunk(line, current_chunk, result)
            continue
        current_chunk.append(line)
    break_off_chunk('', current_chunk, result)
    return result

doctype_re = re.compile('DOCTYPE HTML PUBLIC',re.IGNORECASE)
xml_version_re = re.compile('xml version=',re.IGNORECASE)
url_line_re = re.compile("""\"?http://""",re.IGNORECASE)

filter_res = [doctype_re, xml_version_re, url_line_re]
#filter_res = [doctype_re]

def filter_strange(line):
    for reg_ex in filter_res:
        m =reg_ex.match(line)
        if m:
            return True
    return False

        



if __name__ == '__main__':
    import sys
    import crawling_utils
    filename = None
    if len(sys.argv) == 2:
        filename = sys.argv[1]
    else:
        print 'Usage: %s <filename>' % sys.argv[0]
    home = os.environ['HOME']
    rel_data_dir = 'Desktop/SWIDSAS/whooping_cough/classes5/vaccinenewsdaily.com/news'
    if filename is None:
        print 'Using default filename'
        filename = os.path.join(home,rel_data_dir, '229411-michigan-sees-pertussis-cases-spike.txt')
        print filename
    # os.chdir(os.path.join(home,rel_data_dir))
    file_list = clean_and_chunk(filename)
        
        
