#! /usr/bin/env python
import xlrd, codecs

xls_exts = ['.xls', '.xlsx']

def make_format_dict (format_line):
    """
    Format line is a line of tab-separated field names; Generally it is the
    first line in a full-format Excel file.
    
    Required fields are required, but there can be any number of other
    fields. So make and return a dictionary indexing the position of the required
    fields.
    """
    required_fields  = ['URL', 'Title', 'IP', 'Host']
    format_dict = {}
    for (index,field) in enumerate(format_line.strip().split('\t')):
                format_dict[field] = index
    for f in required_fields:
        try:
            format_dict[f]
        except KeyError:
            print 'Spread sheet missing format line or required fields'
            print 'Required Fields:  URL Title IP Host Organization'
            raise Exception
    if not ('Organization' in format_dict):
        print 'Warning: Recommended field "Organization" missing!'
    return format_dict

def read_spread_sheet_line (line_list, format,format_dict):
    """
    formats::
        -- 'full'   Full format case (Ming's keyword spread sheet format)
        -- 'abbr'   Abbreviated spread sheet (Tulasi)
        -- 'simple' just the URL
        
    Essentially a line parser.
    
    Given a list of elements, use format_dict to identify which
    fields each element fills and return (page,title, ip_address,host,org).
    """
    # All spread sheet formats tab separated
    if format == 'full':
        try:
            page = line_list[format_dict['URL']]
            title = line_list[format_dict['Title']]
            ip_address = line_list[format_dict['IP']]
            host = line_list[format_dict['Host']]
        except KeyError:
            print 'Spread sheet missing format line or required fields'
            print 'Required Fields:  URL Title IP Host Organization'
            raise Exception
        except IndexError:
            print 'Spread sheet line missing a format field'
            print line
            raise Exception
        try:
            org = line_list[format_dict['Organization']]
        except:
            org = None
    elif format == 'abbr':
        try:
            (page,title, ip_address,host, org, visited) = line_list
        except:
            (page,title, ip_address,host,org) = line_list
    elif format == 'simple':
         title,ip_address, host, org = (None,None,None,None)
         (page,) = line_list
    else:
        raise Exception, 'Unknown spread sheet format'
    return (page,title, ip_address,host,org)

def convert_to_text_file(base,ext,format=None):
     """
     Convert xls spreadsheet to txt file with tab-separated
     fields.  Use utf-8 encoding.

     If format = simple, just print out the url. No format line either.
     """
     book = xlrd.open_workbook(base+ext)
     ofh = open(base + '.txt', 'w')
     for ind in range(book.nsheets):
          sh = book.sheet_by_index(ind)
          format_row = sh.row(0)
          format_row = '\t'.join([codecs.utf_8_encode('%s' % (x.value,))[0] for x in format_row])
          format_dict = make_format_dict(format_row)
          url_index  = format_dict['URL']
          for rx in range(sh.nrows):
               # A list of xlrd.Cell instances
               line = sh.row(rx)
               # Convert to list of strings
               line = ['%s' % (x.value,) for x in line]
               if format == 'simple':
                    line = line[url_index:url_index+1] 
               line = '\t'.join(line)
               # UTF-8 encode
               if not(rx == 0 and format == 'simple'):
                    print >> ofh, codecs.utf_8_encode(line)[0]
     ofh.close()
     return base + '.txt'

if __name__ == '__main__':
    import sys
    import os.path
    if len(sys.argv) > 1:
        (base, ext) = os.path.splitext(os.path.abspath(sys.argv[1]))
    else:
        print 'Usage: %s <filename>' % (sys.argv[0],)

    convert_to_text_file(base,ext)
