#!/usr/bin/env python
"""
Clean up html gunk left behind by BeautifulSoup.

Currently handling classic html frags and Java script frags.

Removing long stretches of empty lines BeautifulSoup
creates.

"""
import sys, re, os.path
## Specifying non greedy match (.*? instead of .*)
## to collect as many HTML tags as possible into separate matches
## so as to get accurate scores.
html_re_expr = re.compile(r'</?[(a)(abbr)(acronym)(b)(blockquote)(br)(body)(cite)(code)(del)(dl)(dt)(dd)(div)(em)(frame)(html)(p)(pre)(q)(strike)(strong)(table)(tbody)(td)(tr)(tt)(var)].*?>')

alph = r'[a-zA-Z]'
lit = r'[a-zA-Z\-\_0-9]'
hexadec = r'\#?[0-9A-F]'
#multiline_script_re = re.compile(r'\s*[(BODY)(div)]|([#\.]%s+)\s*\n?\{\n?%s+:\s*.+?\}' % (lit,lit),re.DOTALL)
multiline_script_re = re.compile(r'\{.+\}',re.DOTALL)


start_res = {re.compile(r'(BODY)|\.%s\s*\{?' % (lit,)): \
                  (re.compile(r'\{|(%s+:\s*.+)' % (lit,)), \
                   re.compile(r'\}'))
             }



########
####   Clean up lines
#########


def clean_up_lines (lines):
    """
    Input a line iterable (can be file handle or list).
    Preserve sequences of empty lines only as singleton
    empty lines.

    Look for multi line filterable patterns using line buffer.

    Get rid of html chunks.
    """
    global line_buffer
    blank_line = False
    lines_out = [ ]
    line_buffer = LineBuffer(start_res)
    for line in lines:
        line = line.strip()
        if line and line_buffer.on():
            if line_buffer.end_line_buffer(line):
                line_buffer.flush()
            elif line_buffer.continue_line_buffer(line):
                line_buffer.add(line)
            else:
                line_buffer.empty(lines_out)
        elif line:
            start_re = line_buffer.start_line_buffer(line)
            if start_re:
                line_buffer.turn_on(start_re,line)
            else:
                line = filter_menu_lines(line)
                lines_out.append(line)
                # print >> o_fh, line
                blank_line = False
        elif blank_line:
            pass
        else:
            #print >> o_fh
            lines_out.append('')
            blank_line = True
    
    return remove_html_lines(lines_out)


def remove_html_lines (lines):
    """
    lines is a sequence of lines. Return same, filtered.
    """
    linestr = '\n'.join(lines)
    linestr = re.sub(html_re_expr, '', linestr)
    linestr = re.sub(multiline_script_re, '',linestr)
    line_list = [line.strip() for line in linestr.split('\n')]
    print 'remove html lines: ', len(line_list)
    return line_list



#####  
#####  utility stuff

def all_capitalized(line):
    word_list = line.split()
    for w in word_list:
        if w.capitalize() == w:
            pass
        else:
            return 0
    return 1

def vertical_bar (line):
    if line.strip().endswith('|'):
        return 1
    else:
        return 0


filter_tests = [all_capitalized, vertical_bar]


def filter_menu_lines(line):
    threshhold = len(filter_tests) - 1
    score = 0
    for test in filter_tests:
        score += test(line)
    if score > threshhold:
        return ''
    else:
        return line





threshhold = 5

def contains_html (data_file,verbose=False):
    match_list = get_match_list(data_file)
    match_ct = len(m_list)
    if match_ct > threshhold:
        print 'HTML score: %s: %d' % (data_file,match_ct)
        if verbose:
            for m in match_list:
                print m
        return 'html'

def get_match_list (data_file):
    global m_list
    lines = "".join(open(data_file,'r').readlines())
    m_list = re.findall(html_re_expr,lines)
    return m_list

def remove_html (data_file, out_file = None):
    if out_file is None:
        # (base,ext) = os.path.splitext(data_file)
        # out_file = base + '_html_clean' + ext
        out_file = data_file
    # else: print os.path.join(os.getcwd(),out_file)
    fh = open(data_file,'r')
    lines = "".join(fh.readlines())
    fh.close()
    ofh = open(out_file,'w')
    lines = re.sub(html_re_expr, '', lines)
    for regex in start_res:
        if re.search(regex, lines):
            lines = re.sub(multiline_script_re, '', lines)
            break
    print >> ofh, lines
    ofh.close()
    return 'Html removed. Outfile: %s' % (out_file,)

def html_clean_directory (data_dir):
    to_clean = os.listdir(data_dir)
    for f in to_clean:
        if os.path.isfile(os.path.join(data_dir,f)):
            remove_html(f)

class LineBuffer (object):

    """
    Used for searches for multiline patterns.

    Somewhat more customizable than using multiline re's,
    but maybe clunkier.  Allows simpler reg exprs.

    An example of the kind of stuff the re patterns are trying to match
    (primarily Java Script)::

    .ns88_button {
    font-family: Verdana, Arial, Helvetica, sans-serif;
    font-size: 10px; font-weight: bold;
    color: #000000;
    text-decoration: none;
    background-color: #EEEEEE;
    border-color: #FFFFFF #666666 #666666 #FFFFFF;
    border-style: solid;
    border-top-width: 1px;
    border-right-width: 1px;
    border-bottom-width: 1px;
    border-left-width: 1px;
    cursor: hand;
    }

    """

    
    def __init__(self, start_res):
        self.start_res = start_res
        self.initialize_buffer()

    def turn_on(self, start_re, line=None):
        self.__on__ = True
        if line:
            self.lines.append(line)
        self.start_re = start_re


    def add(self, line):
        if self.__on__:
            self.lines.append(line)
        else:
            print "Can't add to unstarted buffer"
            return None

    def on (self):
        return self.__on__

    def flush (self):
        """
        Just flush lines.
        Nowhere to send them.
        """
        self.initialize_buffer()

    def empty (self, bucket):
        """
        Empty lines into C{bucket}, a list.
        """
        bucket.extend(self.lines)
        self.initialize_buffer()


    def initialize_buffer (self):
        self.lines = [ ]
        self.start_re = None
        self.__on__ = False


    def start_line_buffer (self,line):
        """
        Return True of False depending
        on whether one of the the re's
        in start_res has been matched.
        """
        for s_pat in self.start_res:
            if s_pat.match(line):
                return s_pat
        return False

    def continue_line_buffer (self, line):
        if self.__on__:
            if self.start_re:
                (continue_re, end_re) = self.start_res[self.start_re]
                return continue_re.match(line)
            else:
                print '*** No start_re in active line buffer! Why? ***'
        else:
            print "Can't continue unstarted buffer"
            return False

    def end_line_buffer (self, line):
        if self.__on__:
            if self.start_re:
                (continue_re, end_re) = self.start_res[self.start_re]
                return end_re.match(line)
            else:
                print '***  No start_re in active line buffer! Why?  ***'
        else:
            print "Can't end unstarted buffer"
            return False

        


if __name__ == '__main__':
    verbose = False
    out_file = None
    in_file = 'text.txt'
    check = False
    normal_mode = True
    if len(sys.argv) > 2:
        if sys.argv[1] == '-c':
            check = True
            in_file = sys.argv[2]
        else:
            in_file = sys.argv[1]
            out_file = sys.argv[2]
    elif len(sys.argv) > 1:
        in_file = sys.argv[1]
    elif sys.argv:
        print 'Usage: %s [options] <in_file> [<out_file>]' % (sys.argv[0],)
        print 'Options: '
        print '    -c: check file for html'
        print "Using 'test.txt' as default in_file"
    if check:
        verbose = True
        print contains_html(in_file,verbose=verbose)
    elif normal_mode:
        print remove_html(in_file,out_file)
    else:
        home = os.environ['HOME']
        data_dir = os.path.join(home,'Desktop','SWIDSAS','new_data','html_dir')
        print 'Cleaning %s' % (data_dir,)
        html_clean_directory(data_dir)
    
